package tech.nevets.vcardgen;

import org.eclipse.jetty.websocket.api.*;
import org.eclipse.jetty.websocket.api.annotations.*;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;

@WebSocket
public class LiveGenWebSocket {

    private Card card;

    @OnWebSocketConnect
    public void connected(Session session) {}

    @OnWebSocketClose
    public void closed(Session session, int statusCode, String reason) {}

    @OnWebSocketMessage
    public void message(Session session, String message) throws IOException {
        String[] splitMsg = message.split(";");
        String messageType = splitMsg[0];
        String messageContent = splitMsg[1];

        RemoteEndpoint client = session.getRemote();

        switch (messageType) {
            case "start" -> {
                card = new Card();
                client.sendString("success;Card successfully created.");
            }
            case "continue" -> {
                card = Card.CARD_SESSIONS.get(messageContent);
                if (card == null) {
                    card = new Card();
                    client.sendString("notExist;Card with that id does not exist, a new card has been created.");
                } else {
                    client.sendString("success;Card loaded successfully.");
                }
            }
            case "location" -> {
                card.setLocation(Location.getLocation(messageContent));
                client.sendString("background");
                client.sendBytes(byteBufferFromImage(card.getBackground()));
                client.sendString("location");
                client.sendBytes(byteBufferFromImage(card.renderLocationLayer()));
            }
            case "name" -> {

            }
            case "title" -> {

            }
            case "email" -> {

            }
            case "extension" -> {

            }
            case "directNumber" -> {

            }
            case "cellNumber" -> {

            }
            default -> session.getRemote().sendString("Unknown Message Type: " + messageType);
        }
    }

    private ByteBuffer byteBufferFromImage(BufferedImage image) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "png", baos);
        } catch (IOException e) {
            e.printStackTrace();
            return ByteBuffer.wrap(("Error writing image: " + e.getMessage()).getBytes(StandardCharsets.UTF_8));
        }
        return ByteBuffer.wrap(baos.toByteArray());
    }
}
